@@ -78,6 +78,7 @@ Map::Map () :
 	Commands.registerCommandVoid(CMD_MAP_PAUSE, bindFunctionVoid(Map::triggerPause));
 	Commands.registerCommandVoid(CMD_MAP_RESTART, bindFunctionVoid(Map::triggerRestart));
 	Commands.registerCommandVoid(CMD_MAP_DEBUG, bindFunctionVoid(Map::triggerDebug));
+	Commands.registerCommandVoid(CMD_MAP_DUMP, bindFunctionVoid(Map::dump));
 	Commands.registerCommandVoid(CMD_START, bindFunctionVoid(Map::startMap));
 	Commands.registerCommandVoid(CMD_KILL, bindFunctionVoid(Map::killPlayers));
 	Commands.registerCommandVoid(CMD_FINISHMAP, bindFunctionVoid(Map::finishMap));
@@ -90,6 +91,7 @@ Map::~Map ()
 	Commands.removeCommand(CMD_MAP_PAUSE);
 	Commands.removeCommand(CMD_MAP_RESTART);
 	Commands.removeCommand(CMD_MAP_DEBUG);
+	Commands.removeCommand(CMD_MAP_DUMP);
 	Commands.removeCommand(CMD_START);
 	Commands.removeCommand(CMD_KILL);
 	Commands.removeCommand(CMD_FINISHMAP);
@@ -183,6 +185,14 @@ void Map::triggerRestart ()
 	Commands.executeCommandLine(CMD_MAP_START " " + getName());
 }
 
+void Map::dump ()
+{
+	if (!_world)
+		return;
+
+	_world->Dump();
+}
+
 void Map::triggerDebug ()
 {
 	const bool newstate = Config.getDebugRenderer().activate ^ true;
@@ -205,7 +215,7 @@ void Map::render (void *userdata)
 	if (map->_world) {
 		DebugRenderer renderer(map->_pointCount, map->_points, map->_traceCount, map->_traces,  map->_waterIntersectionPoints, Config.getMapDebugRect(), map->_frontend);
 		map->_world->SetDebugDraw(&renderer);
-		map->_world->DrawDebugData();
+		map->_world->DebugDraw();
 		map->_world->SetDebugDraw(nullptr);
 	}
 }
@@ -665,9 +675,9 @@ public:
 	{
 	}
 
-	float32 ReportFixture (b2Fixture* fixture, const b2Vec2& point, const b2Vec2& normal, float32 fraction) override
+	float ReportFixture (b2Fixture* fixture, const b2Vec2& point, const b2Vec2& normal, float fraction) override
 	{
-		IEntity *e = static_cast<IEntity *>(fixture->GetBody()->GetUserData());
+		IEntity *e = reinterpret_cast<IEntity *>(fixture->GetBody()->GetUserData().pointer);
 		if (e && (e->isSolid() || e->isBorder())) {
 			_fraction = fraction;
 			_entity = e;
@@ -890,25 +900,25 @@ void Map::initPhysics ()
 	_borders[BORDER_BOTTOM] = new Border(BorderType::BOTTOM, *this);
 	_borders[BORDER_PLAYER_BOTTOM] = new Border(BorderType::PLAYER_BOTTOM, *this);
 
-	edge.Set(b2Vec2(zeroX, zeroY), b2Vec2(width, zeroY));
-	b2Fixture *top = boxBody->CreateFixture(&fd);
-	top->SetUserData(_borders[BORDER_TOP]);
+	edge.SetTwoSided(b2Vec2(zeroX, zeroY), b2Vec2(width, zeroY));
+	fd.userData.pointer = (uintptr_t)_borders[BORDER_TOP];
+	boxBody->CreateFixture(&fd);
 
-	edge.Set(b2Vec2(zeroX, zeroY), b2Vec2(zeroX, height));
-	b2Fixture *left = boxBody->CreateFixture(&fd);
-	left->SetUserData(_borders[BORDER_LEFT]);
+	edge.SetTwoSided(b2Vec2(zeroX, zeroY), b2Vec2(zeroX, height));
+	fd.userData.pointer = (uintptr_t)_borders[BORDER_LEFT];
+	boxBody->CreateFixture(&fd);
 
-	edge.Set(b2Vec2(width, height), b2Vec2(width, zeroY));
-	b2Fixture *right = boxBody->CreateFixture(&fd);
-	right->SetUserData(_borders[BORDER_RIGHT]);
+	edge.SetTwoSided(b2Vec2(width, height), b2Vec2(width, zeroY));
+	fd.userData.pointer = (uintptr_t)_borders[BORDER_RIGHT];
+	boxBody->CreateFixture(&fd);
 
-	edge.Set(b2Vec2(zeroX, height), b2Vec2(width, height));
-	b2Fixture *bottom = boxBody->CreateFixture(&fd);
-	bottom->SetUserData(_borders[BORDER_BOTTOM]);
+	edge.SetTwoSided(b2Vec2(zeroX, height), b2Vec2(width, height));
+	fd.userData.pointer = (uintptr_t)_borders[BORDER_BOTTOM];
+	boxBody->CreateFixture(&fd);
 
-	edge.Set(b2Vec2(zeroX, height), b2Vec2(width, getMapHeight()));
-	b2Fixture *playerBottom = boxBody->CreateFixture(&fd);
-	playerBottom->SetUserData(_borders[BORDER_PLAYER_BOTTOM]);
+	edge.SetTwoSided(b2Vec2(zeroX, height), b2Vec2(width, getMapHeight()));
+	fd.userData.pointer = (uintptr_t)_borders[BORDER_PLAYER_BOTTOM];
+	boxBody->CreateFixture(&fd);
 
 	initWater();
 }
@@ -1119,12 +1129,12 @@ b2Body* Map::addToWorld (b2FixtureDef &fixtureDef, b2BodyDef &bodyDef, IEntity *
 		fixtureDef.friction = def->friction;
 	}
 
+	bodyDef.userData.pointer = (uintptr_t)entity;
 	b2Body* body = _world->CreateBody(&bodyDef);
-	body->SetUserData(entity);
 
 	if (fixtureDef.shape != nullptr) {
-		b2Fixture* fixture = body->CreateFixture(&fixtureDef);
-		fixture->SetUserData(const_cast<char*>(""));
+		fixtureDef.userData.pointer = (uintptr_t)const_cast<char*>("");
+		body->CreateFixture(&fixtureDef);
 		entity->addBody(body);
 		return body;
 	}
@@ -1160,8 +1170,8 @@ b2Body* Map::addToWorld (b2FixtureDef &fixtureDef, b2BodyDef &bodyDef, IEntity *
 		b2PolygonShape shape;
 		shape.Set(points, vertexCnt);
 		fixtureDef.shape = &shape;
-		b2Fixture* fixture = body->CreateFixture(&fixtureDef);
-		fixture->SetUserData(const_cast<char*>(polygon.userData.c_str()));
+		fixtureDef.userData.pointer = (uintptr_t)const_cast<char*>(polygon.userData.c_str());;
+		body->CreateFixture(&fixtureDef);
 	}
 
 	// create the shape from the sprite definition circles
@@ -1171,8 +1181,8 @@ b2Body* Map::addToWorld (b2FixtureDef &fixtureDef, b2BodyDef &bodyDef, IEntity *
 		shape.m_p = b2Vec2(circle.center.x, circle.center.y);
 		shape.m_radius = circle.radius;
 		fixtureDef.shape = &shape;
-		b2Fixture* fixture = body->CreateFixture(&fixtureDef);
-		fixture->SetUserData(const_cast<char*>(circle.userData.c_str()));
+		fixtureDef.userData.pointer = (uintptr_t)const_cast<char*>(circle.userData.c_str());;
+		body->CreateFixture(&fixtureDef);
 	}
 
 	entity->addBody(body);
@@ -1638,13 +1648,13 @@ void Map::update (uint32_t deltaTime)
 
 bool Map::ShouldCollide (b2Fixture* fixtureA, b2Fixture* fixtureB)
 {
-	IEntity *entity1 = reinterpret_cast<IEntity*>(fixtureA->GetBody()->GetUserData());
-	IEntity *entity2 = reinterpret_cast<IEntity*>(fixtureB->GetBody()->GetUserData());
+	IEntity *entity1 = reinterpret_cast<IEntity*>(fixtureA->GetBody()->GetUserData().pointer);
+	IEntity *entity2 = reinterpret_cast<IEntity*>(fixtureB->GetBody()->GetUserData().pointer);
 
 	if (entity1 == nullptr)
-		entity1 = reinterpret_cast<IEntity*>(fixtureA->GetUserData());
+		entity1 = reinterpret_cast<IEntity*>(fixtureA->GetUserData().pointer);
 	if (entity2 == nullptr)
-		entity2 = reinterpret_cast<IEntity*>(fixtureB->GetUserData());
+		entity2 = reinterpret_cast<IEntity*>(fixtureB->GetUserData().pointer);
 
 	if (entity1 != nullptr && entity2 != nullptr) {
 		const bool shouldCollide = entity1->shouldCollide(entity2) || entity2->shouldCollide(entity1);
@@ -1669,13 +1679,13 @@ void Map::BeginContact (b2Contact* contact)
 {
 	b2Fixture *fixtureA = contact->GetFixtureA();
 	b2Fixture *fixtureB = contact->GetFixtureB();
-	IEntity *entity1 = reinterpret_cast<IEntity*>(fixtureA->GetBody()->GetUserData());
-	IEntity *entity2 = reinterpret_cast<IEntity*>(fixtureB->GetBody()->GetUserData());
+	IEntity *entity1 = reinterpret_cast<IEntity*>(fixtureA->GetBody()->GetUserData().pointer);
+	IEntity *entity2 = reinterpret_cast<IEntity*>(fixtureB->GetBody()->GetUserData().pointer);
 
 	if (entity1 == nullptr)
-		entity1 = reinterpret_cast<IEntity*>(fixtureA->GetUserData());
+		entity1 = reinterpret_cast<IEntity*>(fixtureA->GetUserData().pointer);
 	if (entity2 == nullptr)
-		entity2 = reinterpret_cast<IEntity*>(fixtureB->GetUserData());
+		entity2 = reinterpret_cast<IEntity*>(fixtureB->GetUserData().pointer);
 
 	if (entity1 != nullptr && entity2 != nullptr) {
 		entity1->onContact(contact, entity2);
@@ -1687,13 +1697,13 @@ void Map::EndContact (b2Contact* contact)
 {
 	b2Fixture *fixtureA = contact->GetFixtureA();
 	b2Fixture *fixtureB = contact->GetFixtureB();
-	IEntity *entity1 = reinterpret_cast<IEntity*>(fixtureA->GetBody()->GetUserData());
-	IEntity *entity2 = reinterpret_cast<IEntity*>(fixtureB->GetBody()->GetUserData());
+	IEntity *entity1 = reinterpret_cast<IEntity*>(fixtureA->GetBody()->GetUserData().pointer);
+	IEntity *entity2 = reinterpret_cast<IEntity*>(fixtureB->GetBody()->GetUserData().pointer);
 
 	if (entity1 == nullptr)
-		entity1 = reinterpret_cast<IEntity*>(fixtureA->GetUserData());
+		entity1 = reinterpret_cast<IEntity*>(fixtureA->GetUserData().pointer);
 	if (entity2 == nullptr)
-		entity2 = reinterpret_cast<IEntity*>(fixtureB->GetUserData());
+		entity2 = reinterpret_cast<IEntity*>(fixtureB->GetUserData().pointer);
 
 	if (entity1 != nullptr && entity2 != nullptr) {
 		entity1->endContact(contact, entity2);
@@ -1705,13 +1715,13 @@ void Map::PostSolve (b2Contact* contact, const b2ContactImpulse* impulse)
 {
 	b2Fixture *fixtureA = contact->GetFixtureA();
 	b2Fixture *fixtureB = contact->GetFixtureB();
-	IEntity *entity1 = reinterpret_cast<IEntity*>(fixtureA->GetBody()->GetUserData());
-	IEntity *entity2 = reinterpret_cast<IEntity*>(fixtureB->GetBody()->GetUserData());
+	IEntity *entity1 = reinterpret_cast<IEntity*>(fixtureA->GetBody()->GetUserData().pointer);
+	IEntity *entity2 = reinterpret_cast<IEntity*>(fixtureB->GetBody()->GetUserData().pointer);
 
 	if (entity1 == nullptr)
-		entity1 = reinterpret_cast<IEntity*>(fixtureA->GetUserData());
+		entity1 = reinterpret_cast<IEntity*>(fixtureA->GetUserData().pointer);
 	if (entity2 == nullptr)
-		entity2 = reinterpret_cast<IEntity*>(fixtureB->GetUserData());
+		entity2 = reinterpret_cast<IEntity*>(fixtureB->GetUserData().pointer);
 
 	if (entity1 != nullptr && entity2 != nullptr) {
 		entity1->onPostSolve(contact, impulse, entity2);
@@ -1723,13 +1733,13 @@ void Map::PreSolve (b2Contact* contact, const b2Manifold* oldManifold)
 {
 	b2Fixture *fixtureA = contact->GetFixtureA();
 	b2Fixture *fixtureB = contact->GetFixtureB();
-	IEntity *entity1 = reinterpret_cast<IEntity*>(fixtureA->GetBody()->GetUserData());
-	IEntity *entity2 = reinterpret_cast<IEntity*>(fixtureB->GetBody()->GetUserData());
+	IEntity *entity1 = reinterpret_cast<IEntity*>(fixtureA->GetBody()->GetUserData().pointer);
+	IEntity *entity2 = reinterpret_cast<IEntity*>(fixtureB->GetBody()->GetUserData().pointer);
 
 	if (entity1 == nullptr)
-		entity1 = reinterpret_cast<IEntity*>(fixtureA->GetUserData());
+		entity1 = reinterpret_cast<IEntity*>(fixtureA->GetUserData().pointer);
 	if (entity2 == nullptr)
-		entity2 = reinterpret_cast<IEntity*>(fixtureB->GetUserData());
+		entity2 = reinterpret_cast<IEntity*>(fixtureB->GetUserData().pointer);
 
 	if (entity1 != nullptr && entity2 != nullptr) {
 		entity1->onPreSolve(contact, entity2, oldManifold);
